@@ -33,6 +33,8 @@
 #include "spa_comm.h"
 #include "utils.h"
 #include "getpasswd.h"
+#include <sys/stat.h>
+#include <fcntl.h>
 
 /* prototypes
 */
@@ -55,7 +57,7 @@ main(int argc, char **argv)
 {
     fko_ctx_t           ctx, ctx2;
     int                 res;
-    char               *spa_data, *version;
+    char               *spa_data;
     char                access_buf[MAX_LINE_LEN];
 
     fko_cli_options_t   options;
@@ -73,6 +75,14 @@ main(int argc, char **argv)
     else if (!options.no_save_args)
         save_args(argc, argv);
 
+    /* Display version info and exit.
+    */
+    if (options.version) {
+        fprintf(stdout, "fwknop client %s, FKO protocol version %s\n",
+            MY_VERSION, FKO_PROTOCOL_VERSION);
+        return(EXIT_SUCCESS);
+    }
+
     /* Intialize the context
     */
     res = fko_new(&ctx);
@@ -82,18 +92,6 @@ main(int argc, char **argv)
         return(EXIT_FAILURE);
     }
 
-    /* Display version info and exit.
-    */
-    if (options.version) {
-        fko_get_version(ctx, &version);
-
-        fprintf(stdout, "fwknop client %s, FKO protocol version %s\n",
-            MY_VERSION, version);
-
-        fko_destroy(ctx);
-        return(EXIT_SUCCESS);
-    }
-
     /* Set client timeout
     */
     if(options.fw_timeout >= 0)
@@ -530,11 +528,14 @@ get_save_file(char *args_save_file)
     char *homedir = NULL;
     int rv = 0;
 
+#ifdef WIN32
+    homedir = getenv("USERPROFILE");
+#else
     homedir = getenv("HOME");
-
+#endif
     if (homedir != NULL) {
-        snprintf(args_save_file, MAX_PATH_LEN, "%s%s%s",
-            homedir, "/", ".fwknop.run");
+        snprintf(args_save_file, MAX_PATH_LEN, "%s%c%s",
+            homedir, PATH_SEP, ".fwknop.run");
         rv = 1;
     }
 
@@ -550,14 +551,6 @@ show_last_command(void)
     char args_str[MAX_LINE_LEN] = "";
     FILE *args_file_ptr = NULL;
 
-#ifdef WIN32
-    /* Not sure what the right thing is here on Win32, just exit
-     * for now.
-    */
-    fprintf(stderr, "--show-last not implemented on Win32 yet.");
-    exit(EXIT_FAILURE);
-#endif
-
     if (get_save_file(args_save_file)) {
         verify_file_perms_ownership(args_save_file);
         if ((args_file_ptr = fopen(args_save_file, "r")) == NULL) {
@@ -592,14 +585,6 @@ run_last_args(fko_cli_options_t *options)
     char            arg_tmp[MAX_LINE_LEN]  = {0};
     char           *argv_new[MAX_CMDLINE_ARGS];  /* should be way more than enough */
 
-
-#ifdef WIN32
-    /* Not sure what the right thing is here on Win32, just return
-     * for now.
-    */
-    return;
-#endif
-
     if (get_save_file(args_save_file))
     {
         verify_file_perms_ownership(args_save_file);
@@ -660,37 +645,34 @@ save_args(int argc, char **argv)
 {
     char args_save_file[MAX_PATH_LEN];
     char args_str[MAX_LINE_LEN] = "";
-    FILE *args_file_ptr = NULL;
-    int i = 0, args_str_len = 0;
-
-#ifdef WIN32
-    /* Not sure what the right thing is here on Win32, just return
-     * for now.
-    */
-    return;
-#endif
+    int i = 0, args_str_len = 0, args_file_fd = -1;
 
     if (get_save_file(args_save_file)) {
-        if ((args_file_ptr = fopen(args_save_file, "w")) == NULL) {
+        args_file_fd = open(args_save_file, O_WRONLY|O_CREAT, S_IRUSR|S_IWUSR);
+        if (args_file_fd == -1) {
             fprintf(stderr, "Could not open args file: %s\n",
                 args_save_file);
             exit(EXIT_FAILURE);
         }
-        for (i=0; i < argc; i++) {
-            args_str_len += strlen(argv[i]);
-            if (args_str_len >= MAX_PATH_LEN) {
-                fprintf(stderr, "argument string too long, exiting.\n");
-                exit(EXIT_FAILURE);
+        else {
+            for (i=0; i < argc; i++) {
+                args_str_len += strlen(argv[i]);
+                if (args_str_len >= MAX_PATH_LEN) {
+                    fprintf(stderr, "argument string too long, exiting.\n");
+                    exit(EXIT_FAILURE);
+                }
+                strlcat(args_str, argv[i], MAX_PATH_LEN);
+                strlcat(args_str, " ", MAX_PATH_LEN);
             }
-            strlcat(args_str, argv[i], MAX_PATH_LEN);
-            strlcat(args_str, " ", MAX_PATH_LEN);
+            strlcat(args_str, "\n", MAX_PATH_LEN);
+            if(write(args_file_fd, args_str, strlen(args_str))
+                    != strlen(args_str)) {
+                fprintf(stderr,
+                "warning, did not write expected number of bytes to args save file\n");
+            }
+            close(args_file_fd);
         }
-        fprintf(args_file_ptr, "%s\n", args_str);
-        fclose(args_file_ptr);
     }
-
-    set_file_perms(args_save_file);
-
     return;
 }
 
