@@ -32,8 +32,6 @@
 #define FKO_H 1
 
 #include <time.h>
-#include "fko_limits.h"
-#include "fko_message.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -55,7 +53,7 @@ extern "C" {
 
 /* General params
 */
-#define FKO_PROTOCOL_VERSION "2.0.3" /* The fwknop protocol version */
+#define FKO_PROTOCOL_VERSION "2.0.4" /* The fwknop protocol version */
 
 /* Supported FKO Message types...
 */
