@@ -53,11 +53,11 @@ _rijndael_encrypt(fko_ctx_t ctx, const char *enc_key)
     /* Make a bucket big enough to hold the enc msg + digest (plaintext)
      * and populate it appropriately.
     */
-    plain = malloc(strlen(ctx->encoded_msg) + strlen(ctx->digest) + 2);
+    plain = malloc(strlen(ctx->encoded_msg) + strlen(ctx->digest) + 4);
     if(plain == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
-    snprintf(plain, strlen(ctx->encoded_msg) + strlen(ctx->digest) + 2,
+    snprintf(plain, strlen(ctx->encoded_msg) + strlen(ctx->digest) + 4,
             "%s:%s", ctx->encoded_msg, ctx->digest);
 
     /* Make a bucket for the encrypted version and populate it.
@@ -135,7 +135,8 @@ _rijndael_decrypt(fko_ctx_t ctx, const char *dec_key)
     if(cipher == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
-    cipher_len = b64_decode(ctx->encrypted_msg, cipher);
+    if((cipher_len = b64_decode(ctx->encrypted_msg, cipher)) < 0)
+        return(FKO_ERROR_INVALID_DATA);
 
     /* Create a bucket for the plaintext data and decrypt the message
      * data into it.
@@ -285,7 +286,8 @@ gpg_decrypt(fko_ctx_t ctx, const char *dec_key)
     if(cipher == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
-    cipher_len = b64_decode(ctx->encrypted_msg, cipher);
+    if((cipher_len = b64_decode(ctx->encrypted_msg, cipher)) < 0)
+        return(FKO_ERROR_INVALID_DATA);
 
     /* Create a bucket for the plaintext data and decrypt the message
      * data into it.
@@ -374,7 +376,7 @@ fko_encrypt_spa_data(fko_ctx_t ctx, const char *enc_key)
     if(ctx->encoded_msg == NULL || FKO_IS_SPA_DATA_MODIFIED(ctx))
         res = fko_encode_spa_data(ctx);
 
-    if(res)
+    if(res != FKO_SUCCESS)
         return(res);
 
     /* Croak on invalid encoded message as well. At present this is a
