@@ -32,26 +32,12 @@
 #ifndef FKO_MESSAGE_H
 #define FKO_MESSAGE_H 1
 
-#if PLATFORM_OPENBSD
-  #include <sys/types.h>
-  #include <netinet/in.h>
-#else
-  #if HAVE_SYS_SOCKET_H
-    #include <sys/socket.h>
-  #endif
-#endif
-#include <arpa/inet.h>
-
-#define MAX_PROTO_STR_LEN   4  /* tcp, udp, icmp for now */
-#define MAX_PORT_STR_LEN    5
-
 /* SPA message format validation functions.
 */
 int validate_cmd_msg(const char *msg);
 int validate_access_msg(const char *msg);
-int validate_proto_port_spec(const char *msg);
 int validate_nat_access_msg(const char *msg);
-int got_allow_ip(const char *msg);
+int validate_proto_port_spec(const char *msg);
 
 #endif /* FKO_MESSAGE_H */
 
